//
//  UpnpCommand.m
//  uPnp2
//
//  Created by rk on 29/01/11.
//  Copyright 2011 Interchart. All rights reserved.
//

#import "AsynchUpnpCommand.h"
#import	"UpnpAvService.h"


@implementation AsynchUpnpCommand

@synthesize command, params, delegate;

-(GDataXMLElement *)constructSoapEnvelope {
    GDataXMLElement *soapEnvelope = [GDataXMLNode elementWithName:@"s:Envelope"];
    GDataXMLNode *soapNs = [GDataXMLNode namespaceWithName:@"s"
                                                 stringValue:SOAP_NS_URI];
    GDataXMLNode *soapEncoding = [GDataXMLNode attributeWithName:@"s:encodingStyle"
                                                     stringValue:@"http://schemas.xmlsoap.org/soap/encoding/"];
    [soapEnvelope addNamespace:soapNs];
    [soapEnvelope addAttribute:soapEncoding];
    return soapEnvelope;
}

-(GDataXMLElement *)constructSoapBody:(UpnpAvService *)aService {
    GDataXMLElement *soapBody = [GDataXMLNode elementWithName:@"s:Body"];

    NSString *actionNsUri = @"";
    if ([aService isRenderingControl]) {
        actionNsUri = UPNP_RC_NS_URI;
    }
    if ([aService isContentDirectory]) {
        actionNsUri = UPNP_CD_NS_URI;
    }    
    if ([aService isTransport]) {
        actionNsUri = UPNP_AVT_NS_URI;
    }    
        
    GDataXMLNode *actionNs = [GDataXMLNode namespaceWithName:@"u"
                                           stringValue:actionNsUri];
    NSString *actionElementName = [NSString stringWithFormat:@"u:%@", self.command];
    
    GDataXMLElement *action = [GDataXMLElement elementWithName:actionElementName];
    [action addNamespace:actionNs];
    if (self.params) {
        NSArray *keys = [self.params allKeys];
		for (NSString *key in keys) {
            GDataXMLElement *paramElement = 
                [GDataXMLElement elementWithName:key
                                     stringValue:[self.params objectForKey:key]];
            [action addChild:paramElement];
        }
    }
    [soapBody addChild:action];
    
    return soapBody;
}

-(GDataXMLDocument *)constructSoapRequest:(UpnpAvService *)aService {
    GDataXMLElement *newSoapBody = [self constructSoapBody:aService];
    GDataXMLElement *newSoapRequest = [self constructSoapEnvelope];
    [newSoapRequest addChild:newSoapBody];
    GDataXMLDocument *requestDoc = [[GDataXMLDocument alloc] initWithRootElement:newSoapRequest];
    NSLog(@"requestDoc: %@", [Util dataToString:[requestDoc XMLData]]);
    return requestDoc;
}


-(void) executeAtUrl:(NSURL *)actionUrl forService:(id)aService {
	UpnpAvService *service = (UpnpAvService *)aService;
	NSLog(@"%@: actionUrl:%@", self, actionUrl);
	NSMutableURLRequest *request = [NSMutableURLRequest 
									requestWithURL:actionUrl 
									cachePolicy:NSURLRequestReloadIgnoringLocalAndRemoteCacheData 
									timeoutInterval:30];
	[request setHTTPMethod:@"POST"];
	[request setValue:
	 [NSString stringWithFormat:@"%@", @"text/xml; charset=\"utf-8\""]
   forHTTPHeaderField:@"Content-Type"];
	[request setValue:
	 [NSString stringWithFormat:@"%@", @"OS/version UPnP/1.0 product/version"]
   forHTTPHeaderField:@"USER-AGENT"];
	NSString *actionName = command;
	NSString *soapAction = [NSString stringWithFormat:@"\"%@#%@\"", 
							service.serviceType,
							actionName];
	[request setValue:soapAction forHTTPHeaderField:@"SoapAction"];
    
    NSData *soapData = [[self constructSoapRequest:aService] XMLData];
	
	[request setValue:
	 [NSString stringWithFormat:@"%d", [soapData length]]
   forHTTPHeaderField:@"Content-Length"];
	
	[request setHTTPBody:soapData];
	
	NSLog(@"request: %@", request);
	NSLog(@"request method: %@", [request HTTPMethod]);
	NSLog(@"request headers: %@", [request allHTTPHeaderFields]);
	NSLog(@"request body:%.*s", [[request HTTPBody] length], [[request HTTPBody] bytes]);
	[readData release];
	readData = [[NSMutableData alloc] init];
	
	[[NSURLConnection alloc] initWithRequest:request
									delegate:self
							startImmediately:YES];	
	
}


- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	NSLog(@"%@ error: %@", self, error);
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	NSLog(@"%@: response:%@", self, response);
	NSHTTPURLResponse *httpResponse = (NSHTTPURLResponse *)response;
	NSLog(@"  headers: %@", [httpResponse allHeaderFields]);
}
//This method will be called several times as the data arrives
- (void) connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[readData appendData: data];
}

- (void) connectionDidFinishLoading:(NSURLConnection *)connection {
	NSLog(@"readData:'%@', ", [Util dataToString:readData]);
    NSError *error;
    GDataXMLDocument *doc = [[GDataXMLDocument alloc] initWithData:readData 
                                                           options:0 error:&error];
    NSLog(@"doc.rootElement: %@", doc.rootElement);
	
	if ([delegate respondsToSelector:@selector(handleAsynchUpnpCommandResult:)]) {
		[delegate handleAsynchUpnpCommandResult:doc];
	}
	
    [doc release];

    
	[readData release];
	readData = [[NSMutableData alloc] init]; // ready for next read
}

@end
